@@ -1,3 +1,4 @@
+# Copyright (c) OpenMMLab. All rights reserved.
 import os.path as osp
 
 import mmcv
@@ -426,6 +427,10 @@ class LoadPanopticAnnotations(LoadAnnotations):
     def _load_masks_and_semantic_segs(self, results):
         """Private function to load mask and semantic segmentation annotations.
 
+        In gt_semantic_seg, the foreground label is from `0` to
+        `num_things - 1`, the background label is from `num_things` to
+        `num_things + num_stuff - 1`, 255 means the ignored label (`VOID`).
+
         Args:
             results (dict): Result dict from :obj:`mmdet.CustomDataset`.
 
@@ -445,11 +450,11 @@ class LoadPanopticAnnotations(LoadAnnotations):
         pan_png = rgb2id(pan_png)
 
         gt_masks = []
-        gt_seg = np.zeros_like(pan_png)  # 0 as ignore
+        gt_seg = np.zeros_like(pan_png) + 255  # 255 as ignore
 
         for mask_info in results['ann_info']['masks']:
             mask = (pan_png == mask_info['id'])
-            gt_seg = np.where(mask, mask_info['category'] + 1, gt_seg)
+            gt_seg = np.where(mask, mask_info['category'], gt_seg)
 
             # The legal thing masks
             if mask_info.get('is_thing'):
